import json

def get_error_from_response(response:any) -> str:
    response_text:str = response.text
    json_object:any = None
    try:
        json_object = json.loads(response_text)
    except:
        return response_text
    
    if 'message' in json_object:
        return json_object['message']

    if 'error' in json_object:
        return json_object['error']

    return response_text




from celerabitpipelineintegration.util_http import get_error_from_response
from celerabitpipelineintegration.model_credentials import Credentials
from celerabitpipelineintegration.uc_authenticator import Authenticator
from celerabitpipelineintegration.int_main_module import ModuleMain
from celerabitpipelineintegration.util_configuration import Configuration
from celerabitpipelineintegration.util_http_client import HttpClient
from celerabitpipelineintegration.uc_scenario_runner import ScenarioRunner
from celerabitpipelineintegration.model_job import JobStatus
from celerabitpipelineintegration.util_logger import print_debug, print_info
from celerabitpipelineintegration.util_json import is_json
from celerabitpipelineintegration.util_compliance_evaluator import ComplianceEvaluator




import array


VALID_OPERATIONS:array = [
    'authenticate',
    'run',
    'eval-job-results',
    'last-status',
    'version'
]

class ExecutionParams:

    operation:str
    __args__:dict = {}

    def set_arg(self, name:str, value:any = None):
        self.__args__[name] = value if value else name
    
    def get_arg(self, name, default_value:any = None) -> any:
        try:
            return self.__args__[name]
        except Exception as e:
            if str(type(e)) != "<class 'KeyError'>":
                raise e
        return None

    def exists(self, key:str) -> bool:
        value:any = self.get_arg(key)
        return True if value else False

    def exists_swtich(self, switch_key:str) -> bool:
        value:any = self.get_arg(switch_key)
        if not value:
            return False
        return  value == switch_key

    def to_string(self) -> str:
        to_str_args:str = ''
        if self.__args__:
            to_str_args = '\t\t[\n'
            for key in self.__args__:
                to_str_args += '\t\t\t"{}": "{}"\n'.format(key, self.__args__[key])
            to_str_args += '\t\t]'

        to_str:str = """
            "operation": "{}"
            "args":
                {}
        """.format(self.operation, to_str_args)
        return to_str

def __is_switch__(value:str) -> bool:
    if not value:
        return False
    return value.startswith('--') or value.startswith('-')

def __validate_operation__(operation:str):
    filtered:array = filter(lambda x : x == operation,  VALID_OPERATIONS)
    if not filtered or len(list(filtered)) == 0:
        raise Exception('Invalid operation: {}'.format(operation))

def parse_from_args(args:array) -> ExecutionParams:

    if args:
        if len(args) == 0:
            raise Exception('Args not received')

        __validate_operation__(args[0])

        execution_params:ExecutionParams = ExecutionParams()
        execution_params.operation = args[0]


        key:str = None
        value:str = None
        param_index:int = 1
        while param_index < len(args):
            key = args[param_index]
            if __is_switch__(key):
                execution_params.set_arg(key)
            else:
                param_index += 1
                value = args[param_index]
                execution_params.set_arg(key, value)
            param_index += 1

    return execution_params




import urllib.parse

from celerabitpipelineintegration.util_configuration import Configuration
from celerabitpipelineintegration.util_http import get_error_from_response
from celerabitpipelineintegration.util_http_client import HttpClient
from celerabitpipelineintegration.util_logger import print_debug, print_info


class ScenarioStatus:

    __client_name__:str
    __application_name__:str
    __scenario_code__:str
    __token__:str

    def __init__(self, 
                client_name:str, \
                application_name:str, \
                scenario_code:str, \
                token:str
                ) -> None:
        self.__client_name__ = client_name
        self.__application_name__ = application_name
        self.__scenario_code__ = scenario_code
        self.__token__ = token

    def get_last_job(self) -> dict:
        print_info('Getting last job for scenario "{}" ...'.format(self.__scenario_code__))
        url:str = Configuration.instance().get_config_value('CELERABIT-ENDPOINTS', 'get-scenario-last-status')
        url = url.format( \
                    client = urllib.parse.quote(self.__client_name__), \
                    application = urllib.parse.quote(self.__application_name__), \
                    scenario = urllib.parse.quote(self.__scenario_code__) \
                    )
        print_debug('Status url: {}'.format(url))

        http_client:HttpClient = HttpClient(self.__token__)
        response:any = http_client.get(url)

        if response.status_code != 200:
            raise Exception(get_error_from_response(response))

        job_info:any = response.json()
        return job_info


from enum import Enum

from celerabitpipelineintegration.util_logger import print_warn

VALID_JOB_STATUS:list = ["SUCCESS", "FINISHED"]

class JobType(Enum):
    API = "API"
    UX = "UX"
    UNKNOWN = "UNKNOWN"

    def __str__(self) -> str:
        return self.name

DEFAULT_TOLERANCE = {
    JobType.API.value: {
        "latency": 0.8,
        "throughput": 0.5,
        "errors": 0.95,
        "deviation": 0.2
    },
    JobType.UX.value: {
        "performance": "MEDIUM"
    }
}

VALID_COLOR_RANGE_VALUES:list = ["GOOD", "MEDIUM", "BAD"]

class ComplianceEvaluator:

    def __init__(self, job_execution_result:dict, tolerance_str:str) -> None:
        self.job_execution_result = job_execution_result
        self.job_type = self.__get_job_type__()
        self.tolerance = self.__get_tolerance_dict__(tolerance_str, self.job_type)

    def __get_tolerance_dict__(self, tolerance_str:str, job_type:JobType) -> dict:
        if not tolerance_str:
            print_warn(f"Tolerance not received.  Working with default values: {DEFAULT_TOLERANCE[job_type.value]}")
            return DEFAULT_TOLERANCE[job_type.value]
        
        base_tolerance:dict = self.__get_dict_from_tokenized_string__(tolerance_str)
        tolerance:dict = {}

        if job_type == JobType.API:
            tolerance = self.__get_tolerance_dict_for_API_job__(base_tolerance)
        elif job_type == JobType.UX:
            tolerance = self.__get_tolerance_dict_for_UX_job__(base_tolerance)
        
        return tolerance
    
    def __get_tolerance_dict_for_UX_job__(self, base_tolerance:dict) -> dict:
        tolerance:dict = {
            "performance": base_tolerance["performance"] 
                                if "performance" in base_tolerance else DEFAULT_TOLERANCE[JobType.UX.value]["performance"]
        }
        return tolerance

    def __get_tolerance_dict_for_API_job__(self, base_tolerance:dict) -> dict:

        tolerance = {
            "latency": float(base_tolerance["latency"]) 
                            if "latency" in base_tolerance else DEFAULT_TOLERANCE[JobType.API.value]["latency"],
            "throughput": float(base_tolerance["throughput"])
                            if "throughput" in base_tolerance else DEFAULT_TOLERANCE[JobType.API.value]["throughput"],
            "errors": float(base_tolerance["errors"]) 
                            if "errors" in base_tolerance else DEFAULT_TOLERANCE[JobType.API.value]["errors"],
            "deviation": float(base_tolerance["deviation"])
                            if "deviation" in base_tolerance else DEFAULT_TOLERANCE[JobType.API.value]["deviation"]
        }

        return tolerance

    def __get_dict_from_tokenized_string__(self, tokenized_string:str) -> dict:
        if not tokenized_string:
            return {}
        
        dict_object:dict = {}
        tokens:list = tokenized_string.split(',')
        splited_token:list = None

        for token in tokens:
            splited_token = token.strip().split('=')

            if len(splited_token) != 2:
                raise Exception(f'Invalid tokenized string: "{token}".  Full string "{tokenized_string}"')

            dict_object[splited_token[0].strip()] = splited_token[1]

        return dict_object
                    
    def __is_valid_float_compliance_value__(self, value:any) -> bool:
        if not value:
            return False
        try:
            float(value)
        except ValueError:
            return False
        return True

    def __validate_job__(self) -> list:
        result_list:list = []

        if not "status" in self.job_execution_result:
            result_list.append('job without status')
        elif not self.job_execution_result["status"] in VALID_JOB_STATUS:
            result_list.append(f"job status is not in {VALID_JOB_STATUS}")

        if self.job_type == JobType.API:
            
            if  not "complianceLatency" in self.job_execution_result or \
                    not self.__is_valid_float_compliance_value__(self.job_execution_result["complianceLatency"]):
                result_list.append("complianceLatency not received in job_execution_result or has an invalid value")

            if not "complianceThroughput" in self.job_execution_result or \
                    not self.__is_valid_float_compliance_value__(self.job_execution_result["complianceThroughput"]):
                result_list.append("complianceThroughput not received in job_execution_result or has an invalid value")

            if not "complianceErrors" in self.job_execution_result or \
                    not self.__is_valid_float_compliance_value__(self.job_execution_result["complianceErrors"]):
                result_list.append("complianceErrors not received in job_execution_result or has an invalid value")

            if not "complianceDeviation" in self.job_execution_result or \
                    not self.__is_valid_float_compliance_value__(self.job_execution_result["complianceDeviation"]):
                result_list.append("complianceDeviation not received in job_execution_result or has an invalid value")

        elif self.job_type == JobType.UX:

            uxResults:dict = self.job_execution_result["uxResults"]
            if not "performance" in uxResults:
                result_list.append(f"No performance received for job UX in results:\n{uxResults}")
            elif not "color" in uxResults["performance"] or not uxResults["performance"]["color"]:
                result_list.append(f"No color range received for job UX in results:\n{uxResults}")
            elif not uxResults["performance"]["color"] in VALID_COLOR_RANGE_VALUES:
                result_list.append(f'No color range has en invalid value: {uxResults["performance"]["color"]}.  Allowed values: {VALID_COLOR_RANGE_VALUES}')

        return result_list            

    def __get_tolerance_value__(self, name:str) -> any:
        value:any = None

        try:
            value = self.tolerance[name]
        except KeyError:
            raise Exception(f'Tolerance string hasn\'t value "{name}"')
        
        return value

    def __evaluate_job_api__(self) -> list:
        current_value:float = 0.0
        minimum:float = 0.0
        error_messagges:list = []

        validation_list:list = self.__validate_job__()

        if len(validation_list) == 0:

            current_value = float(self.job_execution_result['complianceLatency'])
            minimum = self.__get_tolerance_value__('latency') * 100
            if (current_value < minimum):
                error_messagges.append(f'Latency {current_value} below tolerance {minimum}')

            current_value = float(self.job_execution_result['complianceThroughput'])
            minimum = self.__get_tolerance_value__('throughput') * 100
            if (current_value < minimum):
                error_messagges.append(f'Throughput current_value below tolerance minimum')

            current_value = float(self.job_execution_result['complianceErrors'])
            minimum = self.__get_tolerance_value__('errors') * 100
            if (current_value < minimum):
                error_messagges.append(f'Errors {current_value} below tolerance {minimum}')

            current_value = float(self.job_execution_result['complianceDeviation'])
            minimum = self.__get_tolerance_value__('deviation') * 100
            if (current_value < minimum):
                error_messagges.append(f'Deviation {current_value} below tolerance {minimum}')

        else:
            error_messagges = validation_list

        return error_messagges if len(error_messagges) > 0 else None

    def __evaluate_job_ux__(self) ->list:
        error_messagges:list = []
        validation_list:list = self.__validate_job__()

        if len(validation_list) == 0:
            performance:dict = self.job_execution_result["uxResults"]["performance"]
            color_range:str = performance["color"]
            if color_range.lower() != self.tolerance["performance"].lower():
                error_messagges.append(
                    f'Performance not in range.  Required "{self.tolerance["performance"]}", received "{color_range}"'
                )
        else:
            error_messagges = validation_list

        return error_messagges if len(error_messagges) > 0 else None

    def __is_ux_job__(self) -> bool:
        return "uxResults" in self.job_execution_result
    
    def __is_api_job__(self) -> bool:
        return "kpiLatency" in self.job_execution_result

    def __get_job_type__(self) -> JobType:
        if self.__is_ux_job__():
            return JobType.UX
        elif self.__is_api_job__():
            return JobType.API
        
        raise Exception("Can't define job_type from job_execution_result")

    def evaluate(self) -> list:
        if self.job_type == JobType.API:
            return self.__evaluate_job_api__()
        elif self.job_type == JobType.UX:
            return self.__evaluate_job_ux__()
        raise Exception("Invalid Job Type")



from celerabitpipelineintegration.util_configuration import Configuration


DEBUG_TYPE:str = 'debug'
INFO_TYPE:str = 'info'
WARN_TYPE:str = 'warn'
ERROR_TYPE:str = 'error'

def __print_message__(type:str, *args):
    message:str = type.upper() + ' :: ' + ('\n' if type.strip().lower() == ERROR_TYPE else '')
    for arg in args:
        if isinstance(arg, list):
            message += '['
            for item in arg:
                message += str(item) + ', '
            message = message[0, len(message) - 2]
            message += ']'
        else:
            message += (str(arg) + '\t')

    print(' *-*-*-* {message}'.format(message = message))

def print_debug(*args):
    if  not Configuration.instance().get_config_value('LOGGER', 'quiet') and \
        Configuration.instance().get_config_value('LOGGER', 'level') == 'DEBUG':
        __print_message__(DEBUG_TYPE, args)

def print_info(*args):
    if  not Configuration.instance().get_config_value('LOGGER', 'quiet'):
        __print_message__(INFO_TYPE, args)

def print_warn(*args):
    if  not Configuration.instance().get_config_value('LOGGER', 'quiet'):
        __print_message__(WARN_TYPE, args)

def print_error(*args):
    __print_message__(ERROR_TYPE, args)



import array
import json
from celerabitpipelineintegration.model_credentials import Credentials
from celerabitpipelineintegration.uc_scenario_status import ScenarioStatus
from celerabitpipelineintegration.util_compliance_evaluator import ComplianceEvaluator
from celerabitpipelineintegration.util_configuration import Configuration
from celerabitpipelineintegration.model_execution_params import ExecutionParams, parse_from_args
from celerabitpipelineintegration.uc_authenticator import Authenticator
from celerabitpipelineintegration.uc_scenario_runner import ScenarioRunner
from celerabitpipelineintegration.util_misc import to_float
from celerabitpipelineintegration.version import print_version

class ModuleMain:

    __execution_params__:ExecutionParams = None

    def __is_debug_enabled__(self) -> bool:
        return  self.__execution_params__.exists_swtich('--DEBUG') or \
                self.__execution_params__.exists_swtich('-DEBUG') or \
                self.__execution_params__.exists_swtich('DEBUG')

    def __is_quiet__(self) -> bool:
        return  not (
                    self.__execution_params__.exists_swtich('-v') or \
                    self.__execution_params__.exists_swtich('--v')
                )

    def __configure_logger__(self):
        if self.__execution_params__:
            if (self.__is_debug_enabled__()):
                Configuration.instance().set_config_value('LOGGER', 'level', 'DEBUG')
            if (self.__is_quiet__()):
                Configuration.instance().set_config_value('LOGGER', 'quiet', True)

    def __get_credentials__(self) -> Credentials:
        login:str = self.__execution_params__.get_arg('login')
        password:str = self.__execution_params__.get_arg('password')
        if not login:
            raise Exception('Login not received')
        if not password:
            raise Exception('Password not received')
        return Credentials(login, password)

    def __do_authenticate__(self):
        credentials:Credentials = self.__get_credentials__()
        authenticator:Authenticator = Authenticator()
        token:str = authenticator.authenticate(credentials)
        print(token)

    def __job_info_to_string__(self, job_info:dict) -> str:
        if not job_info:
            return None
        json_string:str = json.dumps(job_info)
        return json_string

    def __do_run__(self):
        token:str = self.__execution_params__.get_arg('token')
        client:str = self.__execution_params__.get_arg('client')
        application:str = self.__execution_params__.get_arg('application')
        scenario:str = self.__execution_params__.get_arg('scenario')
        run_timeout_seconds:int = self.__execution_params__.get_arg('timeout')

        if not token:
            raise Exception('Token not received')
        
        if not client:
            raise Exception('Client not received')

        if not application:
            raise Exception('Application not received')

        if not scenario:
            raise Exception('Scenario not received')

        if not run_timeout_seconds:
            run_timeout_seconds = Configuration.instance().get_config_value('INVOKER', 'default-run-timeout-seconds')

        scenario_runner:ScenarioRunner = ScenarioRunner( \
                                                client_name = client, \
                                                application_name = application, \
                                                scenario_code = scenario, \
                                                token = token,
                                                run_timeout_seconds = run_timeout_seconds
                                            )

        job_info:dict = scenario_runner.run_sync()
        print(self.__job_info_to_string__(job_info))

    def __get_job_execution_result_json__(self, job_execution_result_str:str) -> any:
        job_execution_result:any = None
        try:
            job_execution_result = json.loads(job_execution_result_str)
        except Exception as e:
            raise Exception(f'job-result is not a json object. {e} *-*-* \n{job_execution_result_str}')

        return job_execution_result

    def __do_eval_job_results__(self):
        job_execution_result_str:str = self.__execution_params__.get_arg('job-result')
        tolerance_str:str = self.__execution_params__.get_arg('tolerance')

        if not job_execution_result_str:
            raise Exception('job-result not received')
        
        job_execution_result:any = self.__get_job_execution_result_json__(job_execution_result_str)

        evaluator:ComplianceEvaluator = ComplianceEvaluator(job_execution_result, tolerance_str)
        errors:array = evaluator.evaluate()
        if errors and len(errors) > 0:
            error_message:str = ''
            for error in errors:
                error_message += error + "\n"
            print(error_message)

    def __do_get_last_status__(self):
        token:str = self.__execution_params__.get_arg('token')
        client:str = self.__execution_params__.get_arg('client')
        application:str = self.__execution_params__.get_arg('application')
        scenario:str = self.__execution_params__.get_arg('scenario')

        if not token:
            raise Exception('Token not received')
        
        if not client:
            raise Exception('Client not received')

        if not application:
            raise Exception('Application not received')

        if not scenario:
            raise Exception('Scenario not received')
        
        scenario_status:ScenarioStatus = ScenarioStatus( \
                                                client_name = client, \
                                                application_name = application, \
                                                scenario_code = scenario, \
                                                token = token
                                            )
        

        job_info:dict = scenario_status.get_last_job()
        print(self.__job_info_to_string__(job_info))

    def __show_version__(self):
        print_version()

    def run(self, args:array):
        self.__execution_params__ = parse_from_args(args)
        self.__configure_logger__()

        if self.__execution_params__.operation == 'authenticate':
            self.__do_authenticate__()
        elif self.__execution_params__.operation == 'run':
            self.__do_run__()
        elif self.__execution_params__.operation == 'eval-job-results':
            self.__do_eval_job_results__()
        elif self.__execution_params__.operation == 'last-status':
            self.__do_get_last_status__()
        elif self.__execution_params__.operation == 'version':
            self.__show_version__()



from celerabitpipelineintegration.model_credentials import Credentials
import json

from celerabitpipelineintegration.util_configuration import Configuration
from celerabitpipelineintegration.util_http import get_error_from_response
from celerabitpipelineintegration.util_http_client import HttpClient

class Authenticator:

    def __do_post__(self, credentials:Credentials):
        url:str = Configuration.instance().get_config_value('CELERABIT-ENDPOINTS', 'authenticate')
        data:any = {
            'username': credentials.login,
            'password': credentials.password
        }
        http_client:HttpClient = HttpClient()
        response:any = http_client.post(url, json = data)
        return response

    def __get_token__(self, credentials:Credentials) -> str:
        response = self.__do_post__(credentials)
        if response.status_code != 200:
            raise Exception(get_error_from_response(response))

        json_object:any = None
        try:
            json_object = json.loads(response.text)
        except Exception as e:
            raise Exception('Invalid response object.  {error}. {json_response}'.format(error = e, json_response = response.text))

        if not 'token' in json_object:
            raise Exception('Response does not include a token')

        return json_object['token']

    def authenticate(self, credentials:Credentials) -> str:
        token:str = self.__get_token__(credentials)
        return token


import time
import urllib.parse
from celerabitpipelineintegration.model_job import JobStatus
from celerabitpipelineintegration.util_configuration import Configuration
from celerabitpipelineintegration.util_http import get_error_from_response
from celerabitpipelineintegration.util_http_client import HttpClient
from celerabitpipelineintegration.util_logger import print_debug, print_info

class ScenarioRunner:

    __client_name__:str
    __application_name__:str
    __scenario_code__:str
    __token__:str
    __run_timeout_seconds__:int

    def __init__(   self, \
                    client_name:str, \
                    application_name:str, \
                    scenario_code:str, \
                    token:str, \
                    run_timeout_seconds:int \
                ) -> None:
        self.__client_name__ = client_name
        self.__application_name__ = application_name
        self.__scenario_code__ = scenario_code
        self.__token__ = token
        self.__run_timeout_seconds__ = run_timeout_seconds

    def run_async(self) -> int:
        print_info('Running scenario "{}" ...'.format(self.__scenario_code__))
        url:str = Configuration.instance().get_config_value('CELERABIT-ENDPOINTS', 'run')
        url = url.format( \
                    client = urllib.parse.quote(self.__client_name__), \
                    application = urllib.parse.quote(self.__application_name__), \
                    scenario = urllib.parse.quote(self.__scenario_code__) \
                    )
        print_debug('Runner url: {}'.format(url))
        http_client:HttpClient = HttpClient(self.__token__)
        response:any = http_client.get(url)

        if response.status_code != 200:
            raise Exception(get_error_from_response(response))

        json_response:any = response.json()
        job_id:int = int(json_response['jobId'])
        print_info('Job {} created.'.format(job_id))
        return job_id

    def __is_http_success__(self, code:str) -> bool:
        if not code:
            return False
        return str(code).strip().startswith("2")

    def __get_job_info__(self, job_id:int) -> str:
        print_info('Getting job {} information ...'.format(job_id))
        url:str = Configuration.instance().get_config_value('CELERABIT-ENDPOINTS', 'get-job')
        url = url.format(client = self.__client_name__, application = self.__application_name__, scenario = self.__scenario_code__, job = job_id)
        print_debug('Url to get job information "{}" ...'.format(url))

        http_client:HttpClient = HttpClient(self.__token__)
        response:any = http_client.get(url)

        if not self.__is_http_success__(response.status_code):
            raise Exception(f"===> Error asking for job status: {get_error_from_response(response)}")

        return response.json()

    def __is_final_status__(self, status:str) -> bool:
        if status:
            return status != JobStatus.RUNNING.value
        return True

    def __is_running__(self, job_status:str) -> bool:
        return job_status and job_status == JobStatus.RUNNING.value

    def run_sync(self) -> dict:
        job_status:str = JobStatus.RUNNING.value
        job_info:dict = None
        wait_seconds:int = Configuration.instance().get_config_value('INVOKER', 'wait-to-get-status')

        timeout:float = float(self.__run_timeout_seconds__)
        init_time:float = time.time()
        ellapsed:float = 0

        job_id:int = self.run_async()
        while self.__is_running__(job_status):
            print_debug('Wating ultil job {} finish ...'.format(job_id))
            time.sleep(wait_seconds)
            job_info = self.__get_job_info__(job_id)
            job_status = job_info['status']
            print_info('Job {job_id} status: "{status}"'.format(job_id = job_id, status = job_status))

            ellapsed = time.time() - init_time
            if job_status == JobStatus.RUNNING.value and ellapsed > timeout:
                raise Exception('The job {} did not finish in a reasonable time. The expected timeout was {} seconds, but it took {} seconds' \
                                .format(job_id, timeout, ellapsed))

        return job_info



import requests

from celerabitpipelineintegration.util_configuration import Configuration

class HttpClient:

    __token__:str = None

    def __init__(self, token:str = None):
        self.__token__ = token

    def __get_headers__(self, base_headers:any = None) -> dict:
        new_headers:any = base_headers
        if not new_headers:
            new_headers = {}
        
        new_headers['celerabit-invoker'] = Configuration.instance().get_config_value('INVOKER', 'id')
        
        if self.__token__:
            new_headers['Authorization'] = 'Bearer {}'.format(self.__token__)

        return new_headers

    def post(self, url:str, json:dict = None, data:dict = None, headers:dict = None) -> any:
        headers_to_send:any = self.__get_headers__(headers)
        response:any = requests.post(url, json = json, data = data, headers = headers_to_send)
        return response

    def get(self, url:str, params:dict = None, headers:dict = None) -> any:
        headers_to_send:any = self.__get_headers__(headers)
        response:any = requests.get(url, params = params, headers = headers_to_send)
        return response




class Configuration:

    __instance__ = None

    sections: any = {
        'CELERABIT-ENDPOINTS': {
            'authenticate': 'https://sec-api.celerabit.com/authenticate',
            'run': 'https://app-api.celerabit.com/named/client/{client}/target/{application}/scenario/{scenario}/run',
            'get-job': 'https://app-api.celerabit.com/named/client/{client}/target/{application}/scenario/{scenario}/job/{job}',
            'get-scenario-last-status': 'https://app-api.celerabit.com/named/client/{client}/target/{application}/scenario/{scenario}/last-status'
        },
        'INVOKER': {
            'id': 'celerabitpipelineintegration',
            'wait-to-get-status': 10,
            'default-run-timeout-seconds': 500
        },
        'LOGGER': {
            'level': 'INFO',
            'quiet': False
        },
        'COMPLIANCE': {
            'UX-KPIs': {
                'performance': {
                    'ranges': [
                        {'name': 'low', 'min': 0, 'max': 49},
                        {'name': 'mid', 'min': 50, 'max': 89},
                        {'name': 'high', 'min': 90, 'max': 100}
                    ]
                }
            }
        }
    }

    def __init__(self, config_file: str):
        raise RuntimeError('Call instance() instead')

    @classmethod
    def instance(cls):
        if cls.__instance__ is None:
            cls.__instance__ = cls.__new__(cls)
        return cls.__instance__

    def get_config_value(self, group: str, key: str) -> str:
        value: str = None
        try:
            group_dict: any = self.sections[group]
            if group_dict:
                value = group_dict[key]
        except KeyError:
            pass
        
        return value

    def set_config_value(self, group: str, key: str, value: any):
        if group in self.sections:
            section: any = self.sections[group]
            if key in section:
                section[key] = value



import setuptools

from celerabitpipelineintegration.version import LIB_VERSION

long_description:str = open('README.md').read()

setuptools.setup (
    include_package_data=True,
    name='celerabit-pipeline-integration',
    version=LIB_VERSION,
    description='Celerabit tools to integrate with pipelines',
    long_description = long_description,
    long_description_content_type='text/markdown',
    url='https://git-codecommit.zone.amazonaws.com/v1/repos/celerabit-pipeline-integration-py-package',
    author='Raul A. de Villa C.',
    author_email='raul.devilla@techandsolve.com',
    python_requires='>=3.6',
    package_dir={'':'.'},
    packages=setuptools.find_packages(exclude=["test", "test.*", "*.test"]),
    install_requires=['requests', 'urllib3'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent"
    ],
    license_files=('LICENCE',)
)